@@ -40,8 +40,7 @@ static void format_options_menu(void);          /* format of alignment output */
 static void tree_format_options_menu(void);     /* format of tree output */
 static void ss_options_menu(void);
 static sint secstroutput_options(void);
-static sint read_matrix(char **mattxt, char *matnam, sint matn, short *mat, short *xref);
-static sint read_dnamatrix(char **mattxt, char *matnam, sint matn, short *mat, short *xref);
+static sint read_matrix(char *title,MatMenu menu, char *matnam, sint matn, short *mat, short *xref);
 
 /*
 *	 Global variables
@@ -63,9 +62,10 @@ extern sint 	divergence_cutoff;
 extern sint 	debug;
 extern Boolean 	neg_matrix;
 extern Boolean  quick_pairalign;
-extern Boolean	reset_alignments;		/* DES */
+extern Boolean	reset_alignments_new;		/* DES */
+extern Boolean	reset_alignments_all;		/* DES */
 extern sint 	gap_dist;
-extern Boolean 	no_hyd_penalties, no_pref_penalties;
+extern Boolean 	no_var_penalties, no_hyd_penalties, no_pref_penalties;
 extern sint 	output_order;
 extern sint profile_no;
 extern short 	usermat[], pw_usermat[];
@@ -75,14 +75,15 @@ extern short 	dna_xref[], pw_dna_xref[];
 
 extern Boolean 	lowercase; /* Flag for GDE output - set on comm. line*/
 extern Boolean 	cl_seq_numbers;
-extern Boolean 	output_clustal, output_nbrf, output_phylip, output_gcg, output_gde;
-extern Boolean 	output_tree_clustal, output_tree_phylip, output_tree_distances;
+extern Boolean 	output_clustal, output_nbrf, output_phylip, output_gcg, output_gde, output_nexus;
+extern Boolean 	output_tree_clustal, output_tree_phylip, output_tree_distances,output_tree_nexus;
 extern sint     bootstrap_format;
 extern Boolean 	tossgaps, kimura;
 extern Boolean  percent;
 extern Boolean 	usemenu;
 extern Boolean 	showaln, save_parameters;
 extern Boolean	dnaflag;
+extern Boolean  use_ambiguities;
 
 
 extern char 	hyd_residues[];
@@ -113,13 +114,14 @@ extern sint        strand_end_plus;
 extern sint        helix_end_penalty;
 extern sint        strand_end_penalty;
 
-extern char *matrix_txt[];
-extern char *pw_matrix_txt[];
-extern char *dnamatrix_txt[];
+extern MatMenu matrix_menu;
+extern MatMenu pw_matrix_menu;
+extern MatMenu dnamatrix_menu;
 
 static char phylip_name[FILENAMELEN]="";
 static char clustal_name[FILENAMELEN]="";
 static char dist_name[FILENAMELEN]="";
+static char nexus_name[FILENAMELEN]="";
 static char p1_tree_name[FILENAMELEN]="";
 static char p2_tree_name[FILENAMELEN]="";
 
@@ -178,6 +180,10 @@ void main_menu(void)
 
 		switch(toupper(*lin1)) {
 			case '1': seq_input(FALSE);
+				phylip_name[0]=EOS;
+				clustal_name[0]=EOS;
+				dist_name[0]=EOS;
+				nexus_name[0]=EOS;
 				break;
 			case '2': multiple_align_menu();
 				break;
@@ -240,8 +246,8 @@ static void multiple_align_menu(void)
                                         (!quick_pairalign) ? "SLOW" : "FAST");
         fprintf(stdout,"    5.  Pairwise alignment parameters\n");
         fprintf(stdout,"    6.  Multiple alignment parameters\n\n");
-	fprintf(stdout,"    7.  Reset gaps between alignments?");
-	if(reset_alignments)
+	fprintf(stdout,"    7.  Reset gaps before alignment?");
+	if(reset_alignments_new)
 		fprintf(stdout," = ON\n");
 	else
 		fprintf(stdout," = OFF\n");
@@ -271,7 +277,9 @@ static void multiple_align_menu(void)
             break;
         case '6': multi_menu();
             break;
-	case '7': reset_alignments ^= TRUE;
+	case '7': reset_alignments_new ^= TRUE;
+	    if(reset_alignments_new==TRUE)
+		reset_alignments_all=FALSE;
             break;
         case '8': showaln ^= TRUE;
 	    break;
@@ -592,14 +600,18 @@ static void phylogenetic_tree_menu(void)
         switch(toupper(*lin1))
         {
        	 	case '1': seq_input(FALSE);
+				phylip_name[0]=EOS;
+				clustal_name[0]=EOS;
+				dist_name[0]=EOS;
+				nexus_name[0]=EOS;
          	   	break;
         	case '2': tossgaps ^= TRUE;
           	  	break;
       		case '3': kimura ^= TRUE;;
             		break;
-        	case '4': phylogenetic_tree(phylip_name,clustal_name,dist_name);
+        	case '4': phylogenetic_tree(phylip_name,clustal_name,dist_name,nexus_name);
             		break;
-        	case '5': bootstrap_tree(phylip_name,clustal_name);
+        	case '5': bootstrap_tree(phylip_name,clustal_name,nexus_name);
             		break;
 		case '6': tree_format_options_menu();
 			break;
@@ -648,9 +660,11 @@ static void tree_format_options_menu(void)      /* format of tree output */
 					(!output_tree_clustal)  ? "OFF" : "ON");
 	fprintf(stdout,"     2. Toggle Phylip format tree output     =  %s\n",
 					(!output_tree_phylip)   ? "OFF" : "ON");
-	fprintf(stdout,"     3. Toggle Phylip distance matrix output =  %s\n\n",
+	fprintf(stdout,"     3. Toggle Phylip distance matrix output =  %s\n",
 					(!output_tree_distances)? "OFF" : "ON");
-	fprintf(stdout,"     4. Toggle Phylip bootstrap positions    =  %s\n\n",
+	fprintf(stdout,"     4. Toggle Nexus format tree output      =  %s\n\n",
+					(!output_tree_nexus)? "OFF" : "ON");
+	fprintf(stdout,"     5. Toggle Phylip bootstrap positions    =  %s\n\n",
 (bootstrap_format==BS_NODE_LABELS) ? "NODE LABELS" : "BRANCH LABELS");
 	fprintf(stdout,"\n");
 	fprintf(stdout,"     H. HELP\n\n\n");	
@@ -669,6 +683,9 @@ static void tree_format_options_menu(void)      /* format of tree output */
               			output_tree_distances ^= TRUE;
 			  	break;
 			case '4':
+              			output_tree_nexus ^= TRUE;
+			  	break;
+			case '5':
               			if (bootstrap_format == BS_NODE_LABELS)
 					bootstrap_format = BS_BRANCH_LABELS;
 				else
@@ -719,16 +736,18 @@ static void format_options_menu(void)      /* format of alignment output */
 					(!output_gcg) ? "OFF" : "ON");
 	fprintf(stdout,"     4. Toggle PHYLIP format output      =  %s\n",
 					(!output_phylip) ? "OFF" : "ON");
-	fprintf(stdout,"     5. Toggle GDE format output         =  %s\n\n",
+	fprintf(stdout,"     5. Toggle NEXUS format output       =  %s\n",
+					(!output_nexus) ? "OFF" : "ON");
+	fprintf(stdout,"     6. Toggle GDE format output         =  %s\n\n",
 					(!output_gde) ? "OFF" : "ON");
-	fprintf(stdout,"     6. Toggle GDE output case           =  %s\n",
+	fprintf(stdout,"     7. Toggle GDE output case           =  %s\n",
 					(!lowercase) ? "UPPER" : "LOWER");
-	fprintf(stdout,"     7. Toggle CLUSTALW sequence numbers =  %s\n",
+	fprintf(stdout,"     8. Toggle CLUSTALW sequence numbers =  %s\n",
 					(!cl_seq_numbers) ? "OFF" : "ON");
-	fprintf(stdout,"     8. Toggle output order              =  %s\n\n",
+	fprintf(stdout,"     9. Toggle output order              =  %s\n\n",
 					(output_order==0) ? "INPUT FILE" : "ALIGNED");
-	fprintf(stdout,"     9. Create alignment output file(s) now?\n\n");
-	fprintf(stdout,"     0. Toggle parameter output          = %s\n",
+	fprintf(stdout,"     0. Create alignment output file(s) now?\n\n");
+	fprintf(stdout,"     T. Toggle parameter output          = %s\n",
 					(!save_parameters) ? "OFF" : "ON");
 	fprintf(stdout,"\n");
 	fprintf(stdout,"     H. HELP\n\n\n");	
@@ -750,19 +769,22 @@ static void format_options_menu(void)      /* format of alignment output */
               			output_phylip ^= TRUE;
 			  	break;
 			case '5':
-              			output_gde ^= TRUE;
+              			output_nexus ^= TRUE;
 			  	break;
 			case '6':
-              			lowercase ^= TRUE;
+              			output_gde ^= TRUE;
 			  	break;
 			case '7':
-              			cl_seq_numbers ^= TRUE;
+              			lowercase ^= TRUE;
 			  	break;
 			case '8':
+              			cl_seq_numbers ^= TRUE;
+			  	break;
+			case '9':
                                 if (output_order == INPUT) output_order = ALIGNED;
               			else output_order = INPUT;
 			  	break;
-			case '9':		/* DES */
+			case '0':		/* DES */
 				if(empty) {
 					error("No sequences loaded");
 					break;
@@ -771,7 +793,7 @@ static void format_options_menu(void)      /* format of alignment output */
 				if(!open_alignment_output(path)) break;
 				create_alignment_output(1,nseqs);
 				break;
-        		case '0': save_parameters ^= TRUE;
+        		case 'T': save_parameters ^= TRUE;
 	   			 break;
 			case '?':
 			case 'H':
@@ -843,9 +865,10 @@ static void pair_menu(void)
 		fprintf(stdout,"     1. Gap Open Penalty       :%4.2f\n",pw_go_penalty);
 		fprintf(stdout,"     2. Gap Extension Penalty  :%4.2f\n",pw_ge_penalty);
 		fprintf(stdout,"     3. Protein weight matrix  :%s\n" ,
-                                        pw_matrix_txt[pw_matnum-1]);
-		fprintf(stdout,"     4. DNA weight matrix      :%s\n\n" ,
-                                        dnamatrix_txt[pw_dnamatnum-1]);
+                                        matrix_menu.opt[pw_matnum-1].title);
+		fprintf(stdout,"     4. DNA weight matrix      :%s\n" ,
+                                        dnamatrix_menu.opt[pw_dnamatnum-1].title);
+		fprintf(stdout,"\n");
 
 		fprintf(stdout,"     Fast/Approximate alignments:\n\n");
 
@@ -897,10 +920,10 @@ static void pair_menu(void)
 				pw_ge_penalty=(float)getreal("Enter number",(double)0.0,(double)10.0,(double)pw_ge_penalty);
 				break;
                         case '3':
-                                pw_matnum = read_matrix(pw_matrix_txt,pw_mtrxname,pw_matnum,pw_usermat,pw_aa_xref);
+                                pw_matnum = read_matrix("PROTEIN",pw_matrix_menu,pw_mtrxname,pw_matnum,pw_usermat,pw_aa_xref);
                                 break;
                         case '4':
-                                pw_dnamatnum = read_dnamatrix(dnamatrix_txt,pw_dnamtrxname,pw_dnamatnum,pw_userdnamat,pw_dna_xref);
+                                pw_dnamatnum = read_matrix("DNA",dnamatrix_menu,pw_dnamtrxname,pw_dnamatnum,pw_userdnamat,pw_dna_xref);
                                 break;
 			case '5':
                                 fprintf(stdout,"Gap Penalty Currently: %d\n",(pint)wind_gap);
@@ -978,9 +1001,9 @@ static void multi_menu(void)
 
                 fprintf(stdout,"     4. DNA Transitions Weight           :%1.2f\n\n",transition_weight);
                 fprintf(stdout,"     5. Protein weight matrix            :%s\n"
-                                        	,matrix_txt[matnum-1]);
+                                        	,matrix_menu.opt[matnum-1].title);
                 fprintf(stdout,"     6. DNA weight matrix                :%s\n"
-                                        	,dnamatrix_txt[dnamatnum-1]);
+                                        	,dnamatrix_menu.opt[dnamatnum-1].title);
 		fprintf(stdout,"     7. Use negative matrix              :%s\n\n",(!neg_matrix) ? "OFF" : "ON");
                 fprintf(stdout,"     8. Protein Gap Parameters\n\n");
 		fprintf(stdout,"     H. HELP\n\n\n");		
@@ -1017,10 +1040,10 @@ static void multi_menu(void)
 				transition_weight=(float)getreal("Enter number",(double)0.0,(double)1.0,(double)transition_weight);
 				break;
 			case '5':
-                                matnum = read_matrix(matrix_txt,mtrxname,matnum,usermat,aa_xref);
+                                matnum = read_matrix("PROTEIN",matrix_menu,mtrxname,matnum,usermat,aa_xref);
 				break;
 			case '6':
-                                dnamatnum = read_dnamatrix(dnamatrix_txt,dnamtrxname,dnamatnum,userdnamat,dna_xref);
+                                dnamatnum = read_matrix("DNA",dnamatrix_menu,dnamtrxname,dnamatnum,userdnamat,dna_xref);
 				break;
 			case '7':
 				neg_matrix ^= TRUE;
@@ -1125,55 +1148,40 @@ static void gap_penalties_menu(void)
 
 
 
-static sint read_matrix(char **mattxt, char *matnam, sint matn, short *mat, short *xref)
+static sint read_matrix(char *title,MatMenu menu, char *matnam, sint matn, short *mat, short *xref)
 {       static char userfile[FILENAMELEN+1];
+	int i;
 
         while(TRUE)
         {
                 fprintf(stdout,"\n\n\n");
-                fprintf(stdout," ********* PROTEIN WEIGHT MATRIX MENU *********\n");
+                fprintf(stdout," ********* %s WEIGHT MATRIX MENU *********\n",title);
                 fprintf(stdout,"\n\n");
 
-
-                fprintf(stdout,"     1. %s\n",mattxt[0]);
-                fprintf(stdout,"     2. %s\n",mattxt[1]);
-                fprintf(stdout,"     3. %s\n",mattxt[2]);
-                fprintf(stdout,"     4. %s\n",mattxt[3]);
-                fprintf(stdout,"     5. %s\n",mattxt[4]);
+		for(i=0;i<menu.noptions;i++)
+                	fprintf(stdout,"     %d. %s\n",i+1,menu.opt[i].title);
                 fprintf(stdout,"     H. HELP\n\n");
                 fprintf(stdout,
-"     -- Current matrix is the %s ",mattxt[matn-1]);
-                if(matn == 5) fprintf(stdout,"(file = %s)",userfile);
+"     -- Current matrix is the %s ",menu.opt[matn-1].title);
+                if(matn == menu.noptions) fprintf(stdout,"(file = %s)",userfile);
                 fprintf(stdout,"--\n");
 
 
                 getstr("\n\nEnter number (or [RETURN] to exit)",lin2);
                 if(*lin2 == EOS) return(matn);
 
+                i=toupper(*lin2)-'0';
+		if(i>0 && i<menu.noptions) {
+                        strcpy(matnam,menu.opt[i-1].string);
+                        matn=i;
+		} else if (i==menu.noptions) {
+                        if(user_mat(userfile, mat, xref)) {
+                              strcpy(matnam,userfile);
+                              matn=i;
+                        }
+		}
+		else
                 switch(toupper(*lin2))  {
-                        case '1':
-                                strcpy(matnam,"blosum");
-                                matn=1;
-                                break;
-                        case '2':
-                                strcpy(matnam,"pam");
-                                matn=2;
-                                break;
-                        case '3':
-                                strcpy(matnam,"gonnet");
-                                matn=3;
-                                break;
-                        case '4':
-                                strcpy(matnam,"id");
-                                matn=4;
-                                break;
-                        case '5':
-                                if(user_mat(userfile, mat, xref))
-                                  {
-                                     strcpy(matnam,userfile);
-                                     matn=5;
-                                  }
-                                break;
                         case '?':
                         case 'H':
                                 get_help('8');
@@ -1186,57 +1194,6 @@ static sint read_matrix(char **mattxt, char *matnam, sint matn, short *mat, shor
 }
 
 
-
-static sint read_dnamatrix(char **mattxt, char *matnam, sint matn, short *mat, short *xref)
-{       static char userfile[FILENAMELEN+1];
-
-        while(TRUE)
-        {
-                fprintf(stdout,"\n\n\n");
-                fprintf(stdout," ********* DNA WEIGHT MATRIX MENU *********\n");
-                fprintf(stdout,"\n\n");
-
-
-                fprintf(stdout,"     1. %s\n",mattxt[0]);
-                fprintf(stdout,"     2. %s\n",mattxt[1]);
-                fprintf(stdout,"     3. %s\n",mattxt[2]);
-                fprintf(stdout,"     H. HELP\n\n");
-                fprintf(stdout,
-"     -- Current matrix is the %s ",mattxt[matn-1]);
-                if(matn == 5) fprintf(stdout,"(file = %s)",userfile);
-                fprintf(stdout,"--\n");
-
-
-                getstr("\n\nEnter number (or [RETURN] to exit)",lin2);
-                if(*lin2 == EOS) return(matn);
-
-                switch(toupper(*lin2))  {
-                        case '1':
-                                strcpy(matnam,"iub");
-                                matn=1;
-                                break;
-                        case '2':
-                                strcpy(matnam,"clustalw");
-                                matn=2;
-                                break;
-                        case '3':
-                                if(user_mat(userfile, mat, xref))
-                                  {
-                                     strcpy(matnam,userfile);
-                                     matn=3;
-                                  }
-                                break;
-                        case '?':
-                        case 'H':
-                                get_help('8');
-                                break;
-                        default:
-                                fprintf(stdout,"\n\nUnrecognised Command\n\n");
-                                break;
-                }
-        }
-}
-
 char prompt_for_yes_no(char *title,char *prompt)
 {
 	char line[80];
