@@ -18,8 +18,13 @@ void distance_matrix_output(FILE *ofile);
 void nj_tree(char **tree_description, FILE *tree);
 void compare_tree(char **tree1, char **tree2, sint *hits, sint n);
 void print_phylip_tree(char **tree_description, FILE *tree, sint bootstrap);
+void print_nexus_tree(char **tree_description, FILE *tree, sint bootstrap);
 sint two_way_split(char **tree_description, FILE *tree, sint start_row, sint flag, sint bootstrap);
+sint two_way_split_nexus(char **tree_description, FILE *tree, sint start_row, sint flag, sint bootstrap);
 void print_tree(char **tree_description, FILE *tree, sint *totals);
+static Boolean is_ambiguity(char c);
+static void overspill_message(sint overspill,sint total_dists);
+
 
 /*
  *   Global variables
@@ -35,6 +40,7 @@ extern Boolean kimura;    /* Use correction for multiple substitutions */
 extern Boolean output_tree_clustal;   /* clustal text output for trees */
 extern Boolean output_tree_phylip;    /* phylip nested parentheses format */
 extern Boolean output_tree_distances; /* phylip distance matrix */
+extern Boolean output_tree_nexus;     /* nexus format tree */
 extern sint    bootstrap_format;      /* bootstrap file format */
 extern Boolean empty;                 /* any sequences in memory? */
 extern Boolean usemenu;   /* interactive (TRUE) or command line (FALSE) */
@@ -45,6 +51,8 @@ extern char **seq_array;   /* the sequences */
 extern char **names;       /* the seq. names */
 extern char seqname[];		/* name of input file */
 extern sint gap_pos1,gap_pos2;
+extern Boolean use_ambiguities;
+extern char *amino_acid_codes;
 
 static double 	*av;
 static double 	*left_branch, *right_branch;
@@ -59,6 +67,7 @@ static sint 	*boot_positions;
 static FILE 	*phylip_phy_tree_file;
 static FILE 	*clustal_phy_tree_file;
 static FILE 	*distances_phy_tree_file;
+static FILE 	*nexus_phy_tree_file;
 static Boolean 	verbose;
 static char 	*tree_gaps;
 static sint first_seq, last_seq;
@@ -67,7 +76,7 @@ static sint first_seq, last_seq;
 extern sint boot_ntrials;		/* number of bootstrap trials */
 extern unsigned sint boot_ran_seed;	/* random number generator seed */
 
-void phylogenetic_tree(char *phylip_name,char *clustal_name,char *dist_name)
+void phylogenetic_tree(char *phylip_name,char *clustal_name,char *dist_name, char *nexus_name)
 /* 
    Calculate a tree using the distances in the nseqs*nseqs array tmat.
    This is the routine for getting the REAL trees after alignment.
@@ -77,6 +86,7 @@ void phylogenetic_tree(char *phylip_name,char *clustal_name,char *dist_name)
 	sint overspill = 0;
 	sint total_dists;
 	static char **standard_tree;
+	static char **save_tree;
 	char lin2[10];
 
 	if(empty) {
@@ -84,7 +94,7 @@ void phylogenetic_tree(char *phylip_name,char *clustal_name,char *dist_name)
 		return;
 	}
 
-	if(nseqs<=3) {
+	if(nseqs<2) {
 		error("Alignment has only %d sequences",nseqs);
 		return;
 	}
@@ -130,6 +140,18 @@ if(output_tree_distances)
         }
 }
 
+if(output_tree_nexus)
+{
+        if (nexus_name[0]!=EOS) {
+                if((nexus_phy_tree_file = open_explicit_file(
+                nexus_name))==NULL) return;
+        }
+        else {
+		if((nexus_phy_tree_file = open_output_file(
+		"\nEnter name for NEXUS tree output file  ",path,
+		nexus_name,"tre")) == NULL) return;
+        }
+}
 	boot_positions = (sint *)ckalloc( (seqlen_array[first_seq]+2) * sizeof (sint) );
 
 	for(j=1; j<=seqlen_array[first_seq]; ++j) 
@@ -151,6 +173,14 @@ if(output_tree_distances)
 			overspill = prot_distance_matrix(phylip_phy_tree_file);
 	}
 
+	if(output_tree_nexus) {
+		verbose = FALSE;     /* Turn off file output */
+		if(dnaflag)
+			overspill = dna_distance_matrix(nexus_phy_tree_file);
+		else 
+			overspill = prot_distance_matrix(nexus_phy_tree_file);
+	}
+
 	if(output_tree_distances) {
 		verbose = FALSE;     /* Turn off file output */
 		if(dnaflag)
@@ -163,18 +193,7 @@ if(output_tree_distances)
 /* check if any distances overflowed the distance corrections */
 	if ( overspill > 0 ) {
 		total_dists = (nseqs*(nseqs-1))/2;
-		fprintf(stdout,"\n");
-		fprintf(stdout,"\n WARNING: %ld of the distances out of a total of %ld",
-		(long)overspill,(long)total_dists);
-		fprintf(stdout,"\n were out of range for the distance correction.");
-		fprintf(stdout,"\n This may not be fatal but you have been warned!");
-		fprintf(stdout,"\n");
-		fprintf(stdout,"\n SUGGESTIONS: 1) turn off the correction");
-		fprintf(stdout,"\n           or 2) remove the most distant sequences");
-		fprintf(stdout,"\n           or 3) use the PHYLIP package.");
-		fprintf(stdout,"\n\n");
-		if (usemenu) 
-			getstr("Press [RETURN] to continue",lin2);
+		overspill_message(overspill,total_dists);
 	}
 
 	if(output_tree_clustal) verbose = TRUE;     /* Turn on file output */
@@ -182,13 +201,27 @@ if(output_tree_distances)
 	standard_tree   = (char **) ckalloc( (nseqs+1) * sizeof (char *) );
 	for(i=0; i<nseqs+1; i++) 
 		standard_tree[i]  = (char *) ckalloc( (nseqs+1) * sizeof(char) );
+	save_tree   = (char **) ckalloc( (nseqs+1) * sizeof (char *) );
+	for(i=0; i<nseqs+1; i++) 
+		save_tree[i]  = (char *) ckalloc( (nseqs+1) * sizeof(char) );
 
-	if(output_tree_clustal || output_tree_phylip) 
+	if(output_tree_clustal || output_tree_phylip || output_tree_nexus) 
 		nj_tree(standard_tree,clustal_phy_tree_file);
 
+	for(i=1; i<nseqs+1; i++) 
+		for(j=1; j<nseqs+1; j++) 
+			save_tree[i][j]  = standard_tree[i][j];
+
 	if(output_tree_phylip) 
 		print_phylip_tree(standard_tree,phylip_phy_tree_file,0);
 
+	for(i=1; i<nseqs+1; i++) 
+		for(j=1; j<nseqs+1; j++) 
+			standard_tree[i][j]  = save_tree[i][j];
+
+	if(output_tree_nexus) 
+		print_nexus_tree(standard_tree,nexus_phy_tree_file,0);
+
 /*
 	print_tree(standard_tree,phy_tree_file);
 */
@@ -198,10 +231,14 @@ if(output_tree_distances)
 	if (right_branch != NULL) right_branch=ckfree((void *)right_branch);
 	if (tkill != NULL) tkill=ckfree((void *)tkill);
 	if (av != NULL) av=ckfree((void *)av);
-	for (i=1;i<nseqs+1;i++)
+	for (i=0;i<nseqs+1;i++)
 		standard_tree[i]=ckfree((void *)standard_tree[i]);
 	standard_tree=ckfree((void *)standard_tree);
 
+	for (i=0;i<nseqs+1;i++)
+		save_tree[i]=ckfree((void *)save_tree[i]);
+	save_tree=ckfree((void *)save_tree);
+
 if(output_tree_clustal) {
 	fclose(clustal_phy_tree_file);	
 	info("Phylogenetic tree file created:   [%s]",clustal_name);
@@ -217,10 +254,31 @@ if(output_tree_distances) {
 	info("Distance matrix  file  created:   [%s]",dist_name);
 }
 
-
+if(output_tree_nexus) {
+	fclose(nexus_phy_tree_file);	
+	info("Nexus tree file  created:   [%s]",nexus_name);
 }
 
 
+}
+
+static void overspill_message(sint overspill,sint total_dists)
+{
+	char err_mess[1024]="";
+
+	sprintf(err_mess,"%d of the distances out of a total of %d",
+	(pint)overspill,(pint)total_dists);
+	strcat(err_mess,"\n were out of range for the distance correction.");
+	strcat(err_mess,"\n");
+	strcat(err_mess,"\n SUGGESTIONS: 1) remove the most distant sequences");
+	strcat(err_mess,"\n           or 2) use the PHYLIP package");
+	strcat(err_mess,"\n           or 3) turn off the correction.");
+	strcat(err_mess,"\n Note: Use option 3 with caution! With this degree");
+	strcat(err_mess,"\n of divergence you will have great difficulty");
+	strcat(err_mess,"\n getting robust and reliable trees.");
+	strcat(err_mess,"\n\n");
+	warning(err_mess);
+}
 
 
 
@@ -229,14 +287,15 @@ Boolean transition(sint base1, sint base2) /* TRUE if transition; else FALSE */
 
    assumes that the bases of DNA sequences have been translated as
    a,A = 0;   c,C = 1;   g,G = 2;   t,T,u,U = 3;  N = 4;  
+   a,A = 0;   c,C = 2;   g,G = 6;   t,T,u,U =17;  
 
    A <--> G  and  T <--> C  are transitions;  all others are transversions.
 
 */
 {
-	if( ((base1 == 0) && (base2 == 2)) || ((base1 == 2) && (base2 == 0)) )
+	if( ((base1 == 0) && (base2 == 6)) || ((base1 == 6) && (base2 == 0)) )
 		return TRUE;                                     /* A <--> G */
-	if( ((base1 == 3) && (base2 == 1)) || ((base1 == 1) && (base2 == 3)) )
+	if( ((base1 ==17) && (base2 == 2)) || ((base1 == 2) && (base2 ==17)) )
 		return TRUE;                                     /* T <--> C */
     return FALSE;
 }
@@ -259,6 +318,7 @@ void tree_gap_delete(void)   /* flag all positions in alignment that have a gap
 			}
 		}
 	}
+
 }
 
 void distance_matrix_output(FILE *ofile)
@@ -305,6 +365,11 @@ void nj_tree(char **tree_description, FILE *tree)
 		fprintf(tree,"\n Numbers in parentheses are branch lengths\n\n");
 	}	
 
+	if (fnseqs == 2) {
+		if (verbose) fprintf(tree,"Cycle   1     =  SEQ:   1 (%9.5f) joins  SEQ:   2 (%9.5f)",tmat[first_seq][first_seq+1],tmat[first_seq][first_seq+1]);
+		return;
+	}
+
 	mini = minj = 0;
 
 	left_branch 	= (double *) ckalloc( (nseqs+2) * sizeof (double)   );
@@ -532,14 +597,16 @@ void nj_tree(char **tree_description, FILE *tree)
 
 
 
-void bootstrap_tree(char *phylip_name,char *clustal_name)
+void bootstrap_tree(char *phylip_name,char *clustal_name, char *nexus_name)
 {
 	sint i,j;
 	int ranno;
 	char path[MAXLINE+1];
     char dummy[10];
+	char err_mess[1024];
 	static char **sample_tree;
 	static char **standard_tree;
+	static char **save_tree;
 	sint total_dists, overspill = 0, total_overspill = 0;
 	sint nfails = 0;
 
@@ -548,13 +615,13 @@ void bootstrap_tree(char *phylip_name,char *clustal_name)
 		return;
 	}
 
-        if(nseqs<=3) {
+        if(nseqs<4) {
                 error("Alignment has only %d sequences",nseqs);
                 return;
         }
 
-	if(!output_tree_clustal && !output_tree_phylip) {
-		error("You must select either clustal or phylip tree output format");
+	if(!output_tree_clustal && !output_tree_phylip && !output_tree_nexus) {
+		error("You must select either clustal or phylip or nexus tree output format");
 		return;
 	}
 	get_path(seqname, path);
@@ -586,6 +653,18 @@ void bootstrap_tree(char *phylip_name,char *clustal_name)
 	}
 	}
 
+	if (output_tree_nexus) {
+        if (nexus_name[0]!=EOS) {
+                if((nexus_phy_tree_file = open_explicit_file(
+                nexus_name))==NULL) return;
+        }
+	else {
+		if((nexus_phy_tree_file = open_output_file(
+		"\nEnter name for bootstrap output file  ",path,
+		nexus_name,"treb")) == NULL) return;
+	}
+	}
+
 	boot_totals    = (sint *)ckalloc( (nseqs+1) * sizeof (sint) );
 	for(i=0;i<nseqs+1;i++)
 		boot_totals[i]=0;
@@ -611,21 +690,18 @@ void bootstrap_tree(char *phylip_name,char *clustal_name)
 			overspill = prot_distance_matrix(phylip_phy_tree_file);
 	}
 
+	if(output_tree_nexus) {
+		verbose = FALSE;     /* Turn off file output */
+		if(dnaflag)
+			overspill = dna_distance_matrix(nexus_phy_tree_file);
+		else 
+			overspill = prot_distance_matrix(nexus_phy_tree_file);
+	}
+
 /* check if any distances overflowed the distance corrections */
 	if ( overspill > 0 ) {
 		total_dists = (nseqs*(nseqs-1))/2;
-		fprintf(stdout,"\n");
-		fprintf(stdout,"\n WARNING: %d of the distances out of a total of %d",
-		(pint)overspill,(pint)total_dists);
-		fprintf(stdout,"\n were out of range for the distance correction.");
-		fprintf(stdout,"\n This may not be fatal but you have been warned!");
-		fprintf(stdout,"\n");
-		fprintf(stdout,"\n SUGGESTIONS: 1) turn off the correction");
-		fprintf(stdout,"\n           or 2) remove the most distant sequences");
-		fprintf(stdout,"\n           or 3) use the PHYLIP package.");
-		fprintf(stdout,"\n\n");
-		if (usemenu) 
-			getstr("Press [RETURN] to continue",dummy);
+		overspill_message(overspill,total_dists);
 	}
 
 	tree_gaps=ckfree((void *)tree_gaps);
@@ -638,7 +714,7 @@ void bootstrap_tree(char *phylip_name,char *clustal_name)
 
 /* compute the standard tree */
 
-	if(output_tree_clustal || output_tree_phylip)
+	if(output_tree_clustal || output_tree_phylip || output_tree_nexus)
 		nj_tree(standard_tree,clustal_phy_tree_file);
 
 	if (output_tree_clustal)
@@ -724,6 +800,13 @@ getint("\n\nEnter number of bootstrap trials ",1,10000,boot_ntrials);
 				overspill = prot_distance_matrix(phylip_phy_tree_file);
 		}
 
+		if(output_tree_nexus) {
+			if(dnaflag)
+				overspill = dna_distance_matrix(nexus_phy_tree_file);
+			else 
+				overspill = prot_distance_matrix(nexus_phy_tree_file);
+		}
+
 		if( overspill > 0) {
 			total_overspill = total_overspill + overspill;
 			nfails++;
@@ -731,7 +814,7 @@ getint("\n\nEnter number of bootstrap trials ",1,10000,boot_ntrials);
 
 		tree_gaps=ckfree((void *)tree_gaps);
 
-		if(output_tree_clustal || output_tree_phylip) 
+		if(output_tree_clustal || output_tree_phylip || output_tree_nexus) 
 			nj_tree(sample_tree,clustal_phy_tree_file);
 
 	 	left_branch=ckfree((void *)left_branch);
@@ -777,6 +860,14 @@ getint("\n\nEnter number of bootstrap trials ",1,10000,boot_ntrials);
 	if (output_tree_clustal)
 		print_tree(standard_tree, clustal_phy_tree_file, boot_totals);
 
+	save_tree   = (char **) ckalloc( (nseqs+1) * sizeof (char *) );
+	for(i=0; i<nseqs+1; i++) 
+		save_tree[i]   = (char *) ckalloc( (nseqs+1) * sizeof(char) );
+
+	for(i=1; i<nseqs+1; i++) 
+		for(j=1; j<nseqs+1; j++) 
+			save_tree[i][j]  = standard_tree[i][j];
+
 	if(output_tree_phylip) {
 		left_branch 	= (double *) ckalloc( (nseqs+2) * sizeof (double)   );
 		right_branch    = (double *) ckalloc( (nseqs+2) * sizeof (double)   );
@@ -790,6 +881,23 @@ getint("\n\nEnter number of bootstrap trials ",1,10000,boot_ntrials);
 		right_branch=ckfree((void *)right_branch);
 	}
 
+	for(i=1; i<nseqs+1; i++) 
+		for(j=1; j<nseqs+1; j++) 
+			standard_tree[i][j]  = save_tree[i][j];
+
+	if(output_tree_nexus) {
+		left_branch 	= (double *) ckalloc( (nseqs+2) * sizeof (double)   );
+		right_branch    = (double *) ckalloc( (nseqs+2) * sizeof (double)   );
+		for (i=1;i<=nseqs;i++) {
+			left_branch[i] = save_left_branch[i];
+			right_branch[i] = save_right_branch[i];
+		}
+		print_nexus_tree(standard_tree,nexus_phy_tree_file,
+						 bootstrap_format);
+		left_branch=ckfree((void *)left_branch);
+		right_branch=ckfree((void *)right_branch);
+	}
+
 	boot_totals=ckfree((void *)boot_totals);
 	save_left_branch=ckfree((void *)save_left_branch);
 	save_right_branch=ckfree((void *)save_right_branch);
@@ -798,18 +906,28 @@ getint("\n\nEnter number of bootstrap trials ",1,10000,boot_ntrials);
 		standard_tree[i]=ckfree((void *)standard_tree[i]);
 	standard_tree=ckfree((void *)standard_tree);
 
+	for (i=0;i<nseqs+1;i++)
+		save_tree[i]=ckfree((void *)save_tree[i]);
+	save_tree=ckfree((void *)save_tree);
+
 	if (output_tree_clustal)
 		fclose(clustal_phy_tree_file);
 
 	if (output_tree_phylip)
 		fclose(phylip_phy_tree_file);
 
+	if (output_tree_nexus)
+		fclose(nexus_phy_tree_file);
+
 	if (output_tree_clustal)
 		info("Bootstrap output file completed       [%s]"
 		,clustal_name);
 	if (output_tree_phylip)
 		info("Bootstrap output file completed       [%s]"
 		,phylip_name);
+	if (output_tree_nexus)
+		info("Bootstrap output file completed       [%s]"
+		,nexus_name);
 }
 
 
@@ -832,10 +950,154 @@ void compare_tree(char **tree1, char **tree2, sint *hits, sint n)
 }
 
 
+void print_nexus_tree(char **tree_description, FILE *tree, sint bootstrap)
+{
+	sint i;
+	sint old_row;
+	
+	fprintf(tree,"#NEXUS\n\n");
+
+	fprintf(tree,"BEGIN TREES;\n\n");
+	fprintf(tree,"\tTRANSLATE\n");
+	for(i=1;i<nseqs;i++) {
+		fprintf(tree,"\t\t%d	%s,\n",(pint)i,names[i]);
+	}
+	fprintf(tree,"\t\t%d	%s\n",(pint)nseqs,names[nseqs]);
+	fprintf(tree,"\t\t;\n");
+
+	fprintf(tree,"\tUTREE PAUP_1= ");
+
+	if(last_seq-first_seq+1==2) {
+		fprintf(tree,"(%d:%7.5f,%d:%7.5f);",first_seq,tmat[first_seq][first_seq+1],first_seq+1,tmat[first_seq][first_seq+1]);
+	}
+	else {
+
+	fprintf(tree,"(");
+ 
+	old_row=two_way_split_nexus(tree_description, tree, last_seq-first_seq+1-2,1,bootstrap);
+	fprintf(tree,":%7.5f",left_branch[last_seq-first_seq+1-2]);
+	if ((bootstrap==BS_BRANCH_LABELS) && (old_row>0) && (boot_totals[old_row]>0))
+		fprintf(tree,"[%d]",(pint)boot_totals[old_row]);
+	fprintf(tree,",");
+
+	old_row=two_way_split_nexus(tree_description, tree, last_seq-first_seq+1-2,2,bootstrap);
+	fprintf(tree,":%7.5f",left_branch[last_seq-first_seq+1-1]);
+	if ((bootstrap==BS_BRANCH_LABELS) && (old_row>0) && (boot_totals[old_row]>0))
+		fprintf(tree,"[%d]",(pint)boot_totals[old_row]);
+	fprintf(tree,",");
+
+	old_row=two_way_split_nexus(tree_description, tree, last_seq-first_seq+1-2,3,bootstrap);
+	fprintf(tree,":%7.5f",left_branch[last_seq-first_seq+1]);
+	if ((bootstrap==BS_BRANCH_LABELS) && (old_row>0) && (boot_totals[old_row]>0))
+		fprintf(tree,"[%d]",(pint)boot_totals[old_row]);
+	fprintf(tree,")");
+        if (bootstrap==BS_NODE_LABELS) fprintf(tree,"TRICHOTOMY");
+	fprintf(tree,";");
+	}
+	fprintf(tree,"\nENDBLOCK;\n");
+}
+
+
+sint two_way_split_nexus
+(char **tree_description, FILE *tree, sint start_row, sint flag, sint bootstrap)
+{
+	sint row, new_row = 0, old_row, col, test_col = 0;
+	Boolean single_seq;
+
+	if(start_row != last_seq-first_seq+1-2) fprintf(tree,"("); 
+
+	for(col=1; col<=last_seq-first_seq+1; col++) {
+		if(tree_description[start_row][col] == flag) {
+			test_col = col;
+			break;
+		}
+	}
+
+	single_seq = TRUE;
+	for(row=start_row-1; row>=1; row--) 
+		if(tree_description[row][test_col] == 1) {
+			single_seq = FALSE;
+			new_row = row;
+			break;
+		}
+
+	if(single_seq) {
+		tree_description[start_row][test_col] = 0;
+		fprintf(tree,"%d",test_col+first_seq-1);
+		if(start_row == last_seq-first_seq+1-2) {
+			return(0);
+		}
+
+		fprintf(tree,":%7.5f,",left_branch[start_row]);
+	}
+	else {
+		for(col=1; col<=last_seq-first_seq+1; col++) {
+		    if((tree_description[start_row][col]==1)&&
+		       (tree_description[new_row][col]==1))
+				tree_description[start_row][col] = 0;
+		}
+		old_row=two_way_split_nexus(tree_description, tree, new_row, (sint)1, bootstrap);
+		if(start_row == last_seq-first_seq+1-2) {
+			return(new_row);
+		}
+
+		fprintf(tree,":%7.5f",left_branch[start_row]);
+		if ((bootstrap==BS_BRANCH_LABELS) && (boot_totals[old_row]>0))
+			fprintf(tree,"[%d]",(pint)boot_totals[old_row]);
+
+		fprintf(tree,",");
+	}
+
+
+	for(col=1; col<=last_seq-first_seq+1; col++) 
+		if(tree_description[start_row][col] == flag) {
+			test_col = col;
+			break;
+		}
+	
+	single_seq = TRUE;
+	new_row = 0;
+	for(row=start_row-1; row>=1; row--) 
+		if(tree_description[row][test_col] == 1) {
+			single_seq = FALSE;
+			new_row = row;
+			break;
+		}
+
+	if(single_seq) {
+		tree_description[start_row][test_col] = 0;
+		fprintf(tree,"%d",test_col+first_seq-1);
+		fprintf(tree,":%7.5f)",right_branch[start_row]);
+	}
+	else {
+		for(col=1; col<=last_seq-first_seq+1; col++) {
+		    if((tree_description[start_row][col]==1)&&
+		       (tree_description[new_row][col]==1))
+				tree_description[start_row][col] = 0;
+		}
+		old_row=two_way_split_nexus(tree_description, tree, new_row, (sint)1, bootstrap);
+		fprintf(tree,":%7.5f",right_branch[start_row]);
+		if ((bootstrap==BS_BRANCH_LABELS) && (boot_totals[old_row]>0))
+			fprintf(tree,"[%d]",(pint)boot_totals[old_row]);
+
+		fprintf(tree,")");
+	}
+	if ((bootstrap==BS_NODE_LABELS) && (boot_totals[start_row]>0))
+			fprintf(tree,"%d",(pint)boot_totals[start_row]);
+	
+	return(start_row);
+}
+
+
 void print_phylip_tree(char **tree_description, FILE *tree, sint bootstrap)
 {
 	sint old_row;
 	
+	if(last_seq-first_seq+1==2) {
+		fprintf(tree,"(%s:%7.5f,%s:%7.5f);",names[first_seq],tmat[first_seq][first_seq+1],names[first_seq+1],tmat[first_seq][first_seq+1]);
+		return;
+	}
+
 	fprintf(tree,"(\n");
  
 	old_row=two_way_split(tree_description, tree, last_seq-first_seq+1-2,1,bootstrap);
@@ -1023,6 +1285,9 @@ sint dna_distance_matrix(FILE *tree)
 			if( (res1 == gap_pos1)     || (res1 == gap_pos2) ||
                             (res2 == gap_pos1) || (res2 == gap_pos2)) 
 				goto skip;          /* gap in a seq*/
+			if(!use_ambiguities)
+			if( is_ambiguity(res1) || is_ambiguity(res2))
+				goto skip;          /* ambiguity code in a seq*/
 			e = e + 1.0;
                         if(res1 != res2) {
 				if(transition(res1,res2))
@@ -1197,12 +1462,19 @@ void guide_tree(FILE *tree,sint firstseq,sint numseqs)
 {
         static char **standard_tree;
         sint i;
+	float dist;
 
 	phylip_phy_tree_file=tree;
         verbose = FALSE;
 	first_seq=firstseq;
 	last_seq=first_seq+numseqs-1;
   
+	if(numseqs==2) {
+		dist=tmat[firstseq][firstseq+1]/2.0;
+		fprintf(tree,"(%s:%0.5f,%s:%0.5f);\n",
+			names[firstseq],dist,names[firstseq+1],dist);
+	}
+	else {
         standard_tree   = (char **) ckalloc( (last_seq-first_seq+2) * sizeof (char *) );
         for(i=0; i<last_seq-first_seq+2; i++)
                 standard_tree[i]  = (char *) ckalloc( (last_seq-first_seq+2) * sizeof(char));
@@ -1211,15 +1483,32 @@ void guide_tree(FILE *tree,sint firstseq,sint numseqs)
 
         print_phylip_tree(standard_tree,phylip_phy_tree_file,0);
 
-        left_branch=ckfree((void *)left_branch);
-        right_branch=ckfree((void *)right_branch);
-        tkill=ckfree((void *)tkill);
-        av=ckfree((void *)av);
+        if(left_branch != NULL) left_branch=ckfree((void *)left_branch);
+        if(right_branch != NULL) right_branch=ckfree((void *)right_branch);
+        if(tkill != NULL) tkill=ckfree((void *)tkill);
+        if(av != NULL) av=ckfree((void *)av);
         for (i=1;i<last_seq-first_seq+2;i++)
                 standard_tree[i]=ckfree((void *)standard_tree[i]);
         standard_tree=ckfree((void *)standard_tree);
-
+	}
         fclose(phylip_phy_tree_file);
 
 }
 
+static Boolean is_ambiguity(char c)
+{
+        int i;
+	char codes[]="ACGTU";
+
+        if(use_ambiguities==TRUE)
+        {
+         return FALSE;
+        }
+
+	for(i=0;i<5;i++)
+        	if(amino_acid_codes[c]==codes[i])
+             	   return FALSE;
+
+        return TRUE;
+}
+
